@@ -1,7 +1,7 @@
 from barril.units import Scalar
 
 
-def testEmptyScalar():
+def testEmptyScalar() -> None:
     scalar = Scalar.CreateEmptyScalar()
     assert not scalar.HasCategory()
     assert scalar == scalar.Copy()
