@@ -1,8 +1,10 @@
-from barril.units import ObtainQuantity, Quantity, UnitDatabase
 import pytest
 
+from barril.units import ObtainQuantity
+from barril.units import UnitDatabase
 
-def testReadOnlyQuantity(unit_database_empty):
+
+def testReadOnlyQuantity(unit_database_empty) -> None:
     unit_database = unit_database_empty
 
     unit_database.AddUnitBase("length", "meters", "m")
@@ -11,14 +13,14 @@ def testReadOnlyQuantity(unit_database_empty):
 
     read_only_quantity = ObtainQuantity("m", "length")
     with pytest.raises(AttributeError):
-        read_only_quantity.SetUnit("cm")
+        read_only_quantity.SetUnit("cm")  # type:ignore[attr-defined]
 
     # When creating a copy of a read only quantity we'll make it not read only anymore!
-    copy = read_only_quantity.MakeCopy(Quantity)
+    copy = read_only_quantity.MakeCopy()
     assert copy.GetUnitDatabase() is unit_database
 
 
-def testMultiplyReadOnlyQuantity():
+def testMultiplyReadOnlyQuantity() -> None:
     """
     Raising an Error when Geometry quantity is read only (...)
 
